@@ -39,10 +39,16 @@
 
 struct SSLConnection *connections;
 
-struct CipherSuite TLS_RSA_WITH_AES_128_CBC_SHA =
-{ 0x00, 0x2F };
-struct CipherSuite TLS_RSA_WITH_AES_256_CBC_SHA =
-{ 0x00, 0x35 };
+struct CipherData ciphers[] = {
+/*  { number, encoder,    ivlen, bits, digest, diglen, mode }, */
+    { 0x002F, ENC_AES,    16, 128, DIG_SHA1,   20, MODE_CBC },   /* TLS_RSA_WITH_AES_128_CBC_SHA     */
+    { 0x0035, ENC_AES256, 16, 256, DIG_SHA1,   20, MODE_CBC },   /* TLS_RSA_WITH_AES_256_CBC_SHA     */
+    { 0x009d, ENC_AES256, 4,  256, DIG_SHA384, 48, MODE_GCM },   /* TLS_RSA_WITH_AES_256_GCM_SHA384  */
+    { 0,      0,          0,  0,   0,          0,  0        }
+};
+
+
+
 
 #define TLS_DEBUG 0
 
@@ -120,64 +126,73 @@ P_hash(const char *digest, unsigned char *dest, int dlen, unsigned char *secret,
 }
 
 int
-PRF(unsigned char *dest, int dlen, unsigned char *pre_master_secret,
+PRF(struct SSLConnection *conn,
+        unsigned char *dest, int dlen, unsigned char *pre_master_secret,
         int plen, unsigned char *label, unsigned char *seed, int slen) {
     int i;
 
-    // Split the secret by half to generate MD5 and SHA secret parts
-    int hplen = plen / 2 + plen % 2;
-    unsigned char md5_secret[hplen];
-    unsigned char sha_secret[hplen];
-    memcpy(md5_secret, pre_master_secret, hplen);
-    memcpy(sha_secret, pre_master_secret + plen / 2, plen / 2);
+    if (conn->version < 3) {
+        // Split the secret by half to generate MD5 and SHA secret parts
+        int hplen = plen / 2 + plen % 2;
+        unsigned char md5_secret[hplen];
+        unsigned char sha_secret[hplen];
+        memcpy(md5_secret, pre_master_secret, hplen);
+        memcpy(sha_secret, pre_master_secret + plen / 2, plen / 2);
 
-    // This vars will store the values of P_MD5 and P_SHA-1
-    unsigned char h_md5[dlen];
-    unsigned char h_sha[dlen];
+        // This vars will store the values of P_MD5 and P_SHA-1
+        unsigned char h_md5[dlen];
+        unsigned char h_sha[dlen];
 
-    // Concatenate given seed to the label to get the final seed
-    int llen = strlen((const char*) label);
-    unsigned char fseed[slen + llen];
-    memcpy(fseed, label, llen);
-    memcpy(fseed + llen, seed, slen);
-    tls_debug_print_hex("hash seed",fseed, slen + llen);
+        // Concatenate given seed to the label to get the final seed
+        int llen = strlen((const char*) label);
+        unsigned char fseed[slen + llen];
+        memcpy(fseed, label, llen);
+        memcpy(fseed + llen, seed, slen);
 
-    // Get enough MD5 and SHA1 data to fill output len
-    P_hash("MD5", h_md5, dlen, pre_master_secret, hplen, fseed, slen + llen);
-    P_hash("SHA1", h_sha, dlen, pre_master_secret + hplen, hplen, fseed, slen + llen);
+        // Get enough MD5 and SHA1 data to fill output len
+        P_hash("MD5", h_md5, dlen, pre_master_secret, hplen, fseed, slen + llen);
+        P_hash("SHA1", h_sha, dlen, pre_master_secret + hplen, hplen, fseed, slen + llen);
 
-    // Final output will be MD5 and SHA1 X-ORed
-    for (i = 0; i < dlen; i++)
-        dest[i] = h_md5[i] ^ h_sha[i];
+        // Final output will be MD5 and SHA1 X-ORed
+        for (i = 0; i < dlen; i++)
+            dest[i] = h_md5[i] ^ h_sha[i];
 
-    tls_debug_print_hex("PRF out", dest, dlen);
-    return dlen;
-}
+    } else {
+        // Concatenate given seed to the label to get the final seed
+        int llen = strlen((const char*) label);
+        unsigned char fseed[slen + llen];
+        memcpy(fseed, label, llen);
+        memcpy(fseed + llen, seed, slen);
 
-int
-PRF12(unsigned char *dest, int dlen, unsigned char *pre_master_secret,
-        int plen, unsigned char *label, unsigned char *seed, int slen) {
+        // Get enough SHA data to fill output len
+        switch (conn->cipher_data.digest) {
+            case DIG_SHA1:
+                P_hash("SHA256", dest, dlen, pre_master_secret, plen, fseed, slen + llen);
+                break;
+            case DIG_SHA256:
+                P_hash("SHA256", dest, dlen, pre_master_secret, plen, fseed, slen + llen);
+                break;
+            case DIG_SHA384:
+                P_hash("SHA384", dest, dlen, pre_master_secret, plen, fseed, slen + llen);
+                break;
+            default:
+                break;
+        }
+    }
 
-    // Concatenate given seed to the label to get the final seed
-    int llen = strlen((const char*) label);
-    unsigned char fseed[slen + llen];
-    memcpy(fseed, label, llen);
-    memcpy(fseed + llen, seed, slen);
-    tls_debug_print_hex("hash seed",fseed, slen + llen);
-
-    // Get enough SHA256 data to fill output len
-    P_hash("SHA256", dest, dlen, pre_master_secret, plen, fseed, slen + llen);
     tls_debug_print_hex("PRF out", dest, dlen);
     return dlen;
 }
 
 struct SSLConnection *
-tls_connection_create(struct in_addr caddr, uint16_t cport, struct in_addr saddr, uint16_t sport) {
+tls_connection_create(struct in_addr caddr, uint16_t cport, struct in_addr saddr, uint16_t sport)
+{
     struct SSLConnection *conn = NULL;
     gnutls_datum_t keycontent = { NULL, 0 };
     FILE *keyfp;
     gnutls_x509_privkey_t spkey;
     size_t br;
+    int ret;
 
     // Allocate memory for this connection
     conn = sng_malloc(sizeof(struct SSLConnection));
@@ -187,13 +202,9 @@ tls_connection_create(struct in_addr caddr, uint16_t cport, struct in_addr saddr
     memcpy(&conn->client_port, &cport, sizeof(uint16_t));
     memcpy(&conn->server_port, &sport, sizeof(uint16_t));
 
-    SSL_library_init();
-    OpenSSL_add_all_algorithms();
+    gnutls_global_init();
 
-    if (!(conn->ssl_ctx = SSL_CTX_new(SSLv23_server_method())))
-        return NULL;
-
-    if (!(conn->ssl = SSL_new(conn->ssl_ctx)))
+    if (gnutls_init(&conn->ssl, GNUTLS_SERVER) < GNUTLS_E_SUCCESS)
         return NULL;
 
     if (!(keyfp = fopen(capture_keyfile(), "rb")))
@@ -207,10 +218,20 @@ tls_connection_create(struct in_addr caddr, uint16_t cport, struct in_addr saddr
     fclose(keyfp);
 
     gnutls_x509_privkey_init(&spkey);
-    gnutls_x509_privkey_import(spkey, &keycontent, GNUTLS_X509_FMT_PEM);
+
+    // Import PEM key data
+    ret = gnutls_x509_privkey_import(spkey, &keycontent, GNUTLS_X509_FMT_PEM);
+    if (ret != GNUTLS_E_SUCCESS)
+        return NULL;
+
     sng_free(keycontent.data);
-    gnutls_privkey_init(&conn->server_private_key);
-    gnutls_privkey_import_x509(conn->server_private_key, spkey, 0);
+
+    // Check this is a valid RSA key
+    if (gnutls_x509_privkey_get_pk_algorithm(spkey) != GNUTLS_PK_RSA)
+        return NULL;
+
+    // Store this key into the connection
+    conn->server_private_key = spkey;
 
     // Add this connection to the list
     conn->next = connections;
@@ -237,8 +258,13 @@ tls_connection_destroy(struct SSLConnection *conn)
     }
 
     // Deallocate connection memory
-    SSL_CTX_free(conn->ssl_ctx);
-    SSL_free(conn->ssl);
+    gnutls_deinit(conn->ssl);
+    sng_free(conn->key_material.client_write_MAC_key);
+    sng_free(conn->key_material.server_write_MAC_key);
+    sng_free(conn->key_material.client_write_IV);
+    sng_free(conn->key_material.server_write_IV);
+    sng_free(conn->key_material.client_write_key);
+    sng_free(conn->key_material.server_write_key);
     sng_free(conn);
 }
 
@@ -255,9 +281,9 @@ tls_check_keyfile(const char *keyfile)
     gnutls_datum_t keycontent = { NULL, 0 };
     FILE *keyfp;
     size_t br;
+    int ret;
 
-    SSL_library_init();
-    OpenSSL_add_all_algorithms();
+    gnutls_global_init();
 
     if (access(capture_keyfile(), R_OK) != 0)
         return 0;
@@ -272,10 +298,24 @@ tls_check_keyfile(const char *keyfile)
     br = fread(keycontent.data, 1, keycontent.size, keyfp);
     fclose(keyfp);
 
-    gnutls_x509_privkey_init(&key);
-    if (gnutls_x509_privkey_import(key, &keycontent, GNUTLS_X509_FMT_PEM) < 0)
+    // Check we have read something from keyfile
+    if (!keycontent.data)
         return 0;
+
+    // Initialize keyfile structure
+    ret = gnutls_x509_privkey_init(&key);
+    if (ret < GNUTLS_E_SUCCESS) {
+        fprintf (stderr, "Error initializing keyfile: %s\n", gnutls_strerror(ret));
+        return 0;
+    }
+
+    // Import RSA keyfile
+    ret = gnutls_x509_privkey_import(key, &keycontent, GNUTLS_X509_FMT_PEM);
     sng_free(keycontent.data);
+    if (ret < GNUTLS_E_SUCCESS) {
+        fprintf (stderr, "Error loading keyfile: %s\n", gnutls_strerror(ret));
+        return 0;
+    }
 
     return 1;
 }
@@ -291,11 +331,16 @@ tls_connection_dir(struct SSLConnection *conn, struct in_addr addr, uint16_t por
 }
 
 struct SSLConnection*
-tls_connection_find(struct in_addr addr, uint16_t port) {
+tls_connection_find(struct in_addr src, uint16_t sport, struct in_addr dst, uint16_t dport) {
     struct SSLConnection *conn;
 
     for (conn = connections; conn; conn = conn->next) {
-        if (tls_connection_dir(conn, addr, port) != -1) {
+        if (tls_connection_dir(conn, src, sport) == 0 &&
+                tls_connection_dir(conn, dst, dport) == 1) {
+            return conn;
+        }
+        if (tls_connection_dir(conn, src, sport) == 1 &&
+                tls_connection_dir(conn, dst, dport) == 0) {
             return conn;
         }
     }
@@ -320,7 +365,7 @@ tls_process_segment(packet_t *packet, struct tcphdr *tcp)
     inet_pton(AF_INET, packet->dst.ip, &ip_dst);
 
     // Try to find a session for this ip
-    if ((conn = tls_connection_find(ip_src, sport))) {
+    if ((conn = tls_connection_find(ip_src, sport, ip_dst, dport))) {
         // Update last connection direction
         conn->direction = tls_connection_dir(conn, ip_src, sport);
 
@@ -338,13 +383,22 @@ tls_process_segment(packet_t *packet, struct tcphdr *tcp)
                 break;
             case TCP_STATE_ACK:
             case TCP_STATE_ESTABLISHED:
-                // Process data segment!
-                if (tls_process_record(conn, payload, size_payload, &out, &outl) == 0) {
-                    if ((int32_t) outl > 0) {
-                        packet_set_payload(packet, out, outl);
-                        packet_set_type(packet, PACKET_SIP_TLS);
-                        return 0;
-                    }
+                // Check if we have a SSLv2 Handshake
+                if(tls_record_handshake_is_ssl2(conn, payload, size_payload)) {
+                    if (tls_process_record_ssl2(conn, payload, size_payload, &out, &outl) != 0)
+                        outl = 0;
+
+                } else {
+                    // Process data segment!
+                    if (tls_process_record(conn, payload, size_payload, &out, &outl) != 0)
+                        outl = 0;
+                }
+
+                // This seems a SIP TLS packet ;-)
+                if ((int32_t) outl > 0) {
+                    packet_set_payload(packet, out, outl);
+                    packet_set_type(packet, PACKET_SIP_TLS);
+                    return 0;
                 }
                 break;
             case TCP_STATE_FIN:
@@ -364,6 +418,74 @@ tls_process_segment(packet_t *packet, struct tcphdr *tcp)
     return 0;
 }
 
+int
+tls_record_handshake_is_ssl2(struct SSLConnection *conn, const uint8_t *payload,
+                   const int len)
+{
+    // This magic belongs to wireshark people <3
+    if (len < 3) return 0;
+    // v2 client hello should start this way
+    if (payload[0] != 0x80) return 0;
+    // v2 client hello msg type
+    if (payload[2] != 0x01) return 0;
+    // Seems SSLv2
+    return 1;
+}
+
+int
+tls_process_record_ssl2(struct SSLConnection *conn, const uint8_t *payload,
+                   const int len, uint8_t **out, uint32_t *outl)
+{
+    int record_len_len;
+    uint16 record_len16;
+    uint24 record_len24;
+    uint8_t record_type;
+    const opaque *fragment;
+    int flen;
+
+    // No record data here!
+    if (len == 0)
+        return 0;
+
+    // Record header length
+    record_len_len = (payload[0] & 0x80) ? 2 : 3;
+
+    // Two bytes SSLv2 record length field
+    if (record_len_len == 2) {
+        record_len16.x[0] = (payload[0] & 0x7f) << 8;
+        record_len16.x[1] = (payload[1]);
+        record_type = payload[2];
+        fragment = payload + 3;
+        flen = UINT16_INT(record_len16) - 1 /* record type */;
+    } else {
+        record_len24.x[0] = (payload[0] & 0x3f) << 8;
+        record_len24.x[1] = payload[1];
+        record_len24.x[2] = payload[2];
+        record_type = payload[3];
+        fragment = payload + 4;
+        flen = UINT24_INT(record_len24) - 1 /* record type */;
+    }
+
+    // We only handle Client Hello handshake SSLv2 records
+    if (record_type == 0x01 && flen > sizeof(struct ClientHelloSSLv2)) {
+        // Client Hello SSLv2
+        struct ClientHelloSSLv2 *clienthello = (struct ClientHelloSSLv2 *) fragment;
+
+        // Store TLS version
+        conn->version = clienthello->client_version.minor;
+
+        // Calculate where client random starts
+        const opaque *random = fragment + sizeof(struct ClientHelloSSLv2)
+                            + UINT16_INT(clienthello->cipherlist_len)
+                            + UINT16_INT(clienthello->sessionid_len);
+
+        // Get Client random
+        memcpy(&conn->client_random, random, sizeof(struct Random));
+    }
+
+    return 0;
+}
+
 int
 tls_process_record(struct SSLConnection *conn, const uint8_t *payload,
                    const int len, uint8_t **out, uint32_t *outl)
@@ -393,7 +515,8 @@ tls_process_record(struct SSLConnection *conn, const uint8_t *payload,
                 break;
             case change_cipher_spec:
                 // From now on, this connection will be encrypted using MasterSecret
-                conn->encrypted = 1;
+                if (conn->client_cipher_ctx && conn->server_cipher_ctx)
+                    conn->encrypted = 1;
                 break;
             case application_data:
                 if (conn->encrypted) {
@@ -441,15 +564,7 @@ tls_process_record_handshake(struct SSLConnection *conn, const opaque *fragment,
                 memcpy(&conn->client_random, &clienthello->random, sizeof(struct Random));
 
                 // Check we have a TLS handshake
-                if (!(clienthello->client_version.major == 0x03)) {
-                    tls_connection_destroy(conn);
-                    return 1;
-                }
-
-                // Only TLS 1.0, 1.1 or 1.2 connections
-                if (clienthello->client_version.minor != 0x01
-                        && clienthello->client_version.minor != 0x02
-                        && clienthello->client_version.minor != 0x03) {
+                if (tls_valid_version(clienthello->client_version) != 0) {
                     tls_connection_destroy(conn);
                     return 1;
                 }
@@ -486,53 +601,94 @@ tls_process_record_handshake(struct SSLConnection *conn, const opaque *fragment,
                 exkeys.data = (unsigned char *)&clientkeyex->exchange_keys;
                 tls_debug_print_hex("exchange keys",exkeys.data, exkeys.size);
 
-                gnutls_privkey_decrypt_data(conn->server_private_key, 0, &exkeys, &pms);
+                tls_privkey_decrypt_data(conn->server_private_key, 0, &exkeys, &pms);
+                if (!pms.data) break;
+
                 memcpy(&conn->pre_master_secret, pms.data, pms.size);
                 tls_debug_print_hex("pre_master_secret", pms.data, pms.size);
-                tls_debug_print_hex("client_random", &conn->client_random, 32);
-                tls_debug_print_hex("server_random", &conn->server_random, 32);
+                tls_debug_print_hex("client_random", &conn->client_random, sizeof(struct Random));
+                tls_debug_print_hex("server_random", &conn->server_random, sizeof(struct Random));
 
                 // Get MasterSecret
                 uint8_t *seed = sng_malloc(sizeof(struct Random) * 2);
                 memcpy(seed, &conn->client_random, sizeof(struct Random));
                 memcpy(seed + sizeof(struct Random), &conn->server_random, sizeof(struct Random));
-                if (conn->version < 3) {
-                    PRF((unsigned char *) &conn->master_secret, sizeof(struct MasterSecret),
-                        (unsigned char *) &conn->pre_master_secret, sizeof(struct PreMasterSecret),
-                        (unsigned char *) "master secret", seed, sizeof(struct Random) * 2);
-                } else {
-                    PRF12((unsigned char *) &conn->master_secret, sizeof(struct MasterSecret),
-                        (unsigned char *) &conn->pre_master_secret, sizeof(struct PreMasterSecret),
-                        (unsigned char *) "master secret", seed, sizeof(struct Random) * 2);
-                }
-                tls_debug_print_hex("master_secret", conn->master_secret.random, 48);
+                PRF(conn, (unsigned char *) &conn->master_secret, sizeof(struct MasterSecret),
+                    (unsigned char *) &conn->pre_master_secret, sizeof(struct PreMasterSecret),
+                    (unsigned char *) "master secret", seed, sizeof(struct Random) * 2);
+
+                tls_debug_print_hex("master_secret", conn->master_secret.random, sizeof(struct MasterSecret));
 
                 memcpy(seed, &conn->server_random, sizeof(struct Random) * 2);
                 memcpy(seed + sizeof(struct Random), &conn->client_random, sizeof(struct Random));
 
+                int key_material_len = 0;
+                key_material_len += conn->cipher_data.diglen * 2;
+                key_material_len += conn->cipher_data.ivblock * 2;
+                key_material_len += conn->cipher_data.bits / 4;
+
                 // Generate MACs, Write Keys and IVs
-                if (conn->version < 3) {
-                    PRF((unsigned char *) &conn->key_material, sizeof(struct tls_data),
-                        (unsigned char *) &conn->master_secret, sizeof(struct MasterSecret),
-                        (unsigned char *) "key expansion", seed, sizeof(struct Random) * 2);
+                uint8_t *key_material = sng_malloc(key_material_len);
+                PRF(conn, (unsigned char *) key_material, key_material_len,
+                    (unsigned char *) &conn->master_secret, sizeof(struct MasterSecret),
+                    (unsigned char *) "key expansion", seed, sizeof(struct Random) * 2);
+
+                // Get write mac keys
+                if (conn->cipher_data.mode == MODE_GCM) {
+                    // AEAD ciphers
+                    conn->key_material.client_write_MAC_key = 0;
+                    conn->key_material.server_write_MAC_key = 0;
                 } else {
-                    PRF12((unsigned char *) &conn->key_material, sizeof(struct tls_data),
-                        (unsigned char *) &conn->master_secret, sizeof(struct MasterSecret),
-                        (unsigned char *) "key expansion", seed, sizeof(struct Random) * 2);
+                    // Copy prf output to ssl connection key material
+                    int mk_len = conn->cipher_data.diglen;
+                    conn->key_material.client_write_MAC_key = sng_malloc(mk_len);
+                    memcpy(conn->key_material.client_write_MAC_key, key_material, mk_len);
+                    tls_debug_print_hex("client_write_MAC_key", key_material, mk_len);
+                    key_material += mk_len;
+                    conn->key_material.server_write_MAC_key = sng_malloc(mk_len);
+                    tls_debug_print_hex("server_write_MAC_key", key_material, mk_len);
+                    memcpy(conn->key_material.server_write_MAC_key, key_material, mk_len);
+                    key_material+=mk_len;
                 }
 
-                tls_debug_print_hex("client_write_MAC_key", conn->key_material.client_write_MAC_key, 20);
-                tls_debug_print_hex("server_write_MAC_key", conn->key_material.server_write_MAC_key, 20);
-                tls_debug_print_hex("client_write_key",     conn->key_material.client_write_key, 32);
-                tls_debug_print_hex("server_write_key",     conn->key_material.server_write_key, 32);
-                tls_debug_print_hex("client_write_IV",      conn->key_material.client_write_IV, 16);
-                tls_debug_print_hex("server_write_IV",      conn->key_material.server_write_IV, 16);
+                // Get write keys
+                int wk_len = conn->cipher_data.bits / 8;
+                conn->key_material.client_write_key = sng_malloc(wk_len);
+                memcpy(conn->key_material.client_write_key, key_material, wk_len);
+                tls_debug_print_hex("client_write_key", key_material, wk_len);
+                key_material+=wk_len;
 
-                // Done with the seed
+                conn->key_material.server_write_key = sng_malloc(wk_len);
+                memcpy(conn->key_material.server_write_key, key_material, wk_len);
+                tls_debug_print_hex("server_write_key", key_material, wk_len);
+                key_material+=wk_len;
+
+                // Get IV blocks
+                conn->key_material.client_write_IV = sng_malloc(conn->cipher_data.ivblock);
+                memcpy(conn->key_material.client_write_IV, key_material, conn->cipher_data.ivblock);
+                tls_debug_print_hex("client_write_IV", key_material,  conn->cipher_data.ivblock);
+                key_material+=conn->cipher_data.ivblock;
+                conn->key_material.server_write_IV = sng_malloc(conn->cipher_data.ivblock);
+                memcpy(conn->key_material.server_write_IV, key_material, conn->cipher_data.ivblock);
+                tls_debug_print_hex("server_write_IV", key_material,  conn->cipher_data.ivblock);
+                /* key_material+=conn->cipher_data.ivblock; */
+
+                // Free temporally allocated memory
                 sng_free(seed);
+                //sng_free(key_material);
+
+                int mode = 0;
+                if (conn->cipher_data.mode == MODE_CBC) {
+                    mode = GCRY_CIPHER_MODE_CBC;
+                } else if (conn->cipher_data.mode == MODE_GCM) {
+                    mode = GCRY_CIPHER_MODE_CTR;
+                } else {
+                    tls_connection_destroy(conn);
+                    return 1;
+                }
 
                 // Create Client decoder
-                gcry_cipher_open(&conn->client_cipher_ctx, conn->ciph, GCRY_CIPHER_MODE_CBC, 0);
+                gcry_cipher_open(&conn->client_cipher_ctx, conn->ciph, mode, 0);
                 gcry_cipher_setkey(conn->client_cipher_ctx,
                                    conn->key_material.client_write_key,
                                    gcry_cipher_get_algo_keylen(conn->ciph));
@@ -541,7 +697,7 @@ tls_process_record_handshake(struct SSLConnection *conn, const opaque *fragment,
                                   gcry_cipher_get_algo_blklen(conn->ciph));
 
                 // Create Server decoder
-                gcry_cipher_open(&conn->server_cipher_ctx, conn->ciph, GCRY_CIPHER_MODE_CBC, 0);
+                gcry_cipher_open(&conn->server_cipher_ctx, conn->ciph, mode, 0);
                 gcry_cipher_setkey(conn->server_cipher_ctx,
                                    conn->key_material.server_write_key,
                                    gcry_cipher_get_algo_keylen(conn->ciph));
@@ -550,7 +706,6 @@ tls_process_record_handshake(struct SSLConnection *conn, const opaque *fragment,
                                   gcry_cipher_get_algo_blklen(conn->ciph));
 
                 break;
-            case new_session_ticket:
             case finished:
                 break;
             default:
@@ -575,6 +730,7 @@ tls_process_record_data(struct SSLConnection *conn, const opaque *fragment,
     gcry_cipher_hd_t *evp;
     uint8_t pad;
     size_t flen = len;
+    uint8_t nonce[16] = { 0 };
 
     tls_debug_print_hex("Ciphertext", fragment, len);
 
@@ -584,11 +740,28 @@ tls_process_record_data(struct SSLConnection *conn, const opaque *fragment,
         evp = &conn->server_cipher_ctx;
     }
 
-    // TLS 1.1 and later extract explicit IV
-    if (conn->version >= 2 && len > 16) {
-        gcry_cipher_setiv(*evp, fragment, 16);
-        flen -= 16;
-        fragment += 16;
+    if (conn->cipher_data.mode == MODE_CBC) {
+        // TLS 1.1 and later extract explicit IV
+        if (conn->version >= 2 && len > 16) {
+            gcry_cipher_setiv(*evp, fragment, 16);
+            flen -= 16;
+            fragment += 16;
+        }
+    }
+
+    if (conn->cipher_data.mode == MODE_GCM) {
+        if (conn->direction == 0) {
+            memcpy(nonce, conn->key_material.client_write_IV, conn->cipher_data.ivblock);
+            memcpy(nonce + conn->cipher_data.ivblock, fragment, 8);
+            nonce[15] = 2;
+        } else {
+            memcpy(nonce, conn->key_material.server_write_IV, conn->cipher_data.ivblock);
+            memcpy(nonce + conn->cipher_data.ivblock, fragment, 8);
+            nonce[15] = 2;
+        }
+        gcry_cipher_setctr(*evp, nonce, sizeof(nonce));
+        flen -= 8;
+        fragment += 8;
     }
 
     size_t dlen = len;
@@ -596,16 +769,27 @@ tls_process_record_data(struct SSLConnection *conn, const opaque *fragment,
     gcry_cipher_decrypt(*evp, decoded, dlen, (void *) fragment, flen);
     tls_debug_print_hex("Plaintext", decoded, flen);
 
-    // Get padding counter and remove from data
-    pad = decoded[flen - 1];
-    dlen = flen - (pad + 1);
-    tls_debug_print_hex("Mac", decoded + (dlen - 20), 20);
+    // Strip mac from the decoded data
+    if (conn->cipher_data.mode == MODE_CBC) {
+        // Get padding counter and remove from data
+        pad = decoded[flen - 1];
+        dlen = flen - (pad + 1);
+        int mac_len = conn->cipher_data.diglen;
+        tls_debug_print_hex("Mac", decoded + (dlen - mac_len), mac_len);
 
-    if ((int32_t)dlen > 0 && dlen <= *outl) {
-        memcpy(*out, decoded, dlen);
-        *outl = dlen - 20 /* Trailing MAC */;
+        if ((int32_t)dlen > 0 && dlen <= *outl) {
+            memcpy(*out, decoded, dlen);
+            *outl = dlen - mac_len /* Trailing MAC */;
+        }
     }
 
+    // Strip auth tag from decoded data
+    if (conn->cipher_data.mode == MODE_GCM) {
+        if ((int32_t)flen > 16) {
+            memcpy(*out, decoded, dlen);
+            *outl = flen - 16;
+        }
+    }
 
     // Clenaup decoded memory
     sng_free(decoded);
@@ -615,15 +799,107 @@ tls_process_record_data(struct SSLConnection *conn, const opaque *fragment,
 int
 tls_connection_load_cipher(struct SSLConnection *conn)
 {
-    if (conn->cipher_suite.cs1 != 0x00)
-        return 1;
+    int i;
+    int ciphnum = (conn->cipher_suite.cs1 << 8) | conn->cipher_suite.cs2;
 
-    if (conn->cipher_suite.cs2 == TLS_RSA_WITH_AES_256_CBC_SHA.cs2) {
-        conn->ciph = gcry_cipher_map_name("AES256");
-    } else if (conn->cipher_suite.cs2 == TLS_RSA_WITH_AES_128_CBC_SHA.cs2) {
-        conn->ciph = gcry_cipher_map_name("AES");
-    } else {
-        return 1;
+    // Check if this is one of the supported ciphers
+    for (i=0; ciphers[i].enc; i++) {
+        if (ciphnum == ciphers[i].num) {
+            conn->cipher_data = ciphers[i];
+            break;
+        }
     }
+
+    // Set proper cipher encoder
+    switch (conn->cipher_data.enc) {
+        case ENC_AES:
+            conn->ciph = gcry_cipher_map_name("AES");
+            break;
+        case ENC_AES256:
+            conn->ciph = gcry_cipher_map_name("AES256");
+            break;
+        default:
+            return 1;
+    }
+
     return 0;
 }
+
+int
+tls_valid_version(struct ProtocolVersion version)
+{
+
+    switch(version.major) {
+        case 0x03:
+            switch(version.minor) {
+                case 0x01:
+                case 0x02:
+                case 0x03:
+                    return 0;
+            }
+    }
+    return 1;
+}
+
+int
+tls_privkey_decrypt_data(gnutls_x509_privkey_t key, unsigned int flags,
+                const gnutls_datum_t * ciphertext, gnutls_datum_t * plaintext)
+{
+    size_t decr_len = 0, i = 0;
+    gcry_sexp_t s_data = NULL, s_plain = NULL;
+    gcry_mpi_t  encr_mpi = NULL, text = NULL;
+    size_t tmp_size;
+    gnutls_datum_t rsa_datum[6];
+    gcry_mpi_t rsa_params[6];
+    gcry_sexp_t rsa_priv_key = NULL;
+
+    // Extract data from RSA key
+    gnutls_x509_privkey_export_rsa_raw(key,
+             &rsa_datum[0], &rsa_datum[1], &rsa_datum[2],
+             &rsa_datum[3], &rsa_datum[4], &rsa_datum[5]);
+
+    // Convert to RSA params
+    for(i=0; i<6; i++) {
+        gcry_mpi_scan(&rsa_params[i], GCRYMPI_FMT_USG, rsa_datum[i].data, rsa_datum[i].size, &tmp_size);
+    }
+
+    if (gcry_mpi_cmp(rsa_params[3], rsa_params[4]) > 0)
+        gcry_mpi_swap(rsa_params[3], rsa_params[4]);
+
+    // Convert to sexp
+    gcry_mpi_invm(rsa_params[5], rsa_params[3], rsa_params[4]);
+    gcry_sexp_build(&rsa_priv_key, NULL,
+        "(private-key(rsa((n%m)(e%m)(d%m)(p%m)(q%m)(u%m))))",
+        rsa_params[0], rsa_params[1], rsa_params[2],
+        rsa_params[3], rsa_params[4], rsa_params[5]);
+
+    // Free not longer required data
+    for (i=0; i< 6; i++) {
+        gcry_mpi_release(rsa_params[i]);
+        gnutls_free(rsa_datum[i].data);
+    }
+
+    gcry_mpi_scan(&encr_mpi, GCRYMPI_FMT_USG, ciphertext->data, ciphertext->size, NULL);
+    gcry_sexp_build(&s_data, NULL, "(enc-val(rsa(a%m)))", encr_mpi);
+    gcry_pk_decrypt(&s_plain, s_data, rsa_priv_key);
+    text = gcry_sexp_nth_mpi(s_plain, 0, 0);
+    gcry_mpi_print(GCRYMPI_FMT_USG, NULL, 0, &decr_len, text);
+    gcry_mpi_print(GCRYMPI_FMT_USG, ciphertext->data, ciphertext->size, &decr_len, text);
+
+    int pad = 0;
+    for (i = 1; i < decr_len; i++) {
+        if (ciphertext->data[i] == 0) {
+            pad = i+1;
+            break;
+        }
+    }
+
+    plaintext->size = decr_len - pad;
+    plaintext->data = gnutls_malloc(plaintext->size);
+    memmove(plaintext->data, ciphertext->data + pad, plaintext->size);
+    gcry_sexp_release(s_data);
+    gcry_sexp_release(s_plain);
+    gcry_mpi_release(encr_mpi);
+    gcry_mpi_release(text);
+    return (int) decr_len;
+}
