@@ -47,9 +47,7 @@
 #define __SNGREP_CAPTURE_TLS_
 
 #include "config.h"
-#include <gnutls/openssl.h>
-#include <gnutls/crypto.h>
-#include <gnutls/abstract.h>
+#include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 #include <gcrypt.h>
 #include "capture.h"
@@ -88,6 +86,25 @@ enum SSLConnectionState {
     TCP_STATE_CLOSED
 };
 
+//! SSL Encoders algo
+enum SSLCipherEncoders {
+    ENC_AES         = 1,
+    ENC_AES256      = 2
+};
+
+//! SSL Digests algo
+enum SSLCIpherDigest {
+    DIG_SHA1        = 1,
+    DIG_SHA256      = 2,
+    DIG_SHA384      = 3
+};
+
+//! SSL Decode mode
+enum SSLCipherMode {
+    MODE_CBC,
+    MODE_GCM
+};
+
 //! ContentType values as defined in RFC5246
 enum ContentType {
     change_cipher_spec  = 20,
@@ -106,7 +123,6 @@ enum HandshakeType {
     server_hello_done   = GNUTLS_HANDSHAKE_SERVER_HELLO_DONE,
     certificate_verify  = GNUTLS_HANDSHAKE_CERTIFICATE_VERIFY,
     client_key_exchange = GNUTLS_HANDSHAKE_CLIENT_KEY_EXCHANGE,
-    new_session_ticket  = GNUTLS_HANDSHAKE_NEW_SESSION_TICKET,
     finished            = GNUTLS_HANDSHAKE_FINISHED
 };
 
@@ -140,6 +156,25 @@ struct CipherSuite {
     uint8_t cs2;
 };
 
+struct CipherData {
+    int num;
+    int enc;
+    int ivblock;
+    int bits;
+    int digest;
+    int diglen;
+    int mode;
+};
+
+struct ClientHelloSSLv2 {
+    struct ProtocolVersion client_version;
+    uint16 cipherlist_len;
+    uint16 sessionid_len;
+    uint16 random_len;
+    // CipherSuite cipher_suite;
+    // struct Random random;
+};
+
 //! ClientHello type in Handshake records
 struct ClientHello {
     struct ProtocolVersion client_version;
@@ -202,23 +237,23 @@ struct SSLConnection {
     //! Server port
     uint16_t server_port;
 
-    SSL *ssl;
-    SSL_CTX *ssl_ctx;
+    gnutls_session_t ssl;
     int ciph;
-    gnutls_privkey_t server_private_key;
+    gnutls_x509_privkey_t server_private_key;
     struct Random client_random;
     struct Random server_random;
     struct CipherSuite cipher_suite;
+    struct CipherData cipher_data;
     struct PreMasterSecret pre_master_secret;
     struct MasterSecret master_secret;
 
     struct tls_data {
-        uint8_t client_write_MAC_key[20];
-        uint8_t server_write_MAC_key[20];
-        uint8_t client_write_key[32];
-        uint8_t server_write_key[32];
-        uint8_t client_write_IV[16];
-        uint8_t server_write_IV[16];
+        uint8_t *client_write_MAC_key;
+        uint8_t *server_write_MAC_key;
+        uint8_t *client_write_key;
+        uint8_t *server_write_key;
+        uint8_t *client_write_IV;
+        uint8_t *server_write_IV;
     } key_material;
 
     gcry_cipher_hd_t client_cipher_ctx;
@@ -247,25 +282,6 @@ int
 P_hash(const char *digest, unsigned char *dest, int dlen, unsigned char *secret, int sslen,
        unsigned char *seed, int slen);
 
-
-/**
- * @brief Pseudorandom Function as defined in RFC5246
- *
- * This function will generate MasterSecret and KeyMaterial data from PreMasterSecret and Seed
- *
- * @param dest Destination of PRF function result. Memory must be already allocated
- * @param dlen Destination length in bytes
- * @param pre_master_secret PreMasterSecret decrypted from ClientKeyExchange Handhsake record
- * @param pslen PreMasterSecret length in bytes
- * @param label Fixed ASCII string
- * @param seed Concatenation of Random data from Hello Handshake records
- * @param slen Seed length in bytes
- * @return destination length in bytes
- */
-int
-PRF12(unsigned char *dest, int dlen, unsigned char *pre_master_secret,
-        int plen, unsigned char *label, unsigned char *seed, int slen);
-
 /**
  * @brief Pseudorandom Function as defined in RFC2246
  *
@@ -281,7 +297,7 @@ PRF12(unsigned char *dest, int dlen, unsigned char *pre_master_secret,
  * @return destination length in bytes
  */
 int
-PRF(unsigned char *dest, int dlen, unsigned char *pre_master_secret,
+PRF(struct SSLConnection *conn, unsigned char *dest, int dlen, unsigned char *pre_master_secret,
         int plen, unsigned char *label, unsigned char *seed, int slen);
 
 /**
@@ -346,7 +362,7 @@ tls_connection_dir(struct SSLConnection *conn, struct in_addr addr, uint16_t por
  * @return an existing Connection pointer or NULL if not found
  */
 struct SSLConnection*
-tls_connection_find(struct in_addr addr, uint16_t port);
+tls_connection_find(struct in_addr src, uint16_t sport, struct in_addr dst, uint16_t dport);
 
 /**
  * @brief Process a TCP segment to check TLS data
@@ -380,6 +396,37 @@ int
 tls_process_record(struct SSLConnection *conn, const uint8_t *payload, const int len, uint8_t **out,
                    uint32_t *outl);
 
+/**
+ * @brief Check if this Record looks like SSLv2
+ *
+ * Some devices send the initial ClientHello in a SSLv2 record for compatibility
+ * with only SSLv2 protocol.
+ *
+ * We will only parse SSLv2 Client hello fragments that have TLSv1/SSLv3 content
+ * so this does not make us SSLv2 compatible ;-p
+ * @param conn Existing connection pointer
+ * @param payload Packet peyload
+ * @param len Payload length
+ * @return 1 if payload seems a SSLv2 record, 0 otherwise
+ */
+int
+tls_record_handshake_is_ssl2(struct SSLConnection *conn, const uint8_t *payload,
+                   const int len);
+/**
+ * @brief Process TLS Handshake SSLv2 record types
+ *
+ * Process all types of Handshake records to store and compute all required
+ * data to decrypt application data packets
+ *
+ * @param conn Existing connection pointer
+ * @param fragment Handshake record data
+ * @param len Decimal length of the fragment
+ * @return 0 on valid record processed, 1 otherwise
+ */
+int
+tls_process_record_ssl2(struct SSLConnection *conn, const uint8_t *payload,
+                   const int len, uint8_t **out, uint32_t *outl);
+
 /**
  * @brief Process TLS Handshake record types
  *
@@ -425,4 +472,31 @@ tls_process_record_data(struct SSLConnection *conn, const opaque *fragment, cons
 int
 tls_connection_load_cipher(struct SSLConnection *conn);
 
+/**
+ * @brief Determine if the given version is valid for us
+ *
+ * We only handle some SSL/TLS versions. This function will filter out
+ * records from unsupported versions.
+ *
+ * @return 0 if the version is supported, 1 otherwise
+ */
+int
+tls_valid_version(struct ProtocolVersion version);
+
+/**
+ * @brief Decrypt data using private RSA key
+ *
+ * This function code has been taken from wireshark.
+ * Because wireshark simply rocks.
+ *
+ * @param key Imported RSA key data
+ * @param flags decrpyt flag (no used)
+ * @param ciphertext Encrypted data
+ * @param plaintext Decrypted data
+ * @return number of bytes of decrypted data
+ */
+int
+tls_privkey_decrypt_data(gnutls_x509_privkey_t key, unsigned int flags,
+                const gnutls_datum_t * ciphertext, gnutls_datum_t * plaintext);
+
 #endif
