@@ -196,10 +196,6 @@ call_flow_draw(ui_t *ui)
         if (vector_iterator_current(&it) == info->first_arrow) {
             info->scroll.pos = info->scroll.max;
         }
-        // Skip RTP arrows if not displayed
-        if (arrow->type == CF_ARROW_RTP && setting_disabled(SETTING_CF_MEDIA)) {
-            continue;
-        }
         info->scroll.max += call_flow_arrow_height(ui, arrow);
     }
     ui_scrollbar_draw(info->scroll);
@@ -267,8 +263,9 @@ call_flow_draw_columns(ui_t *ui)
     if (!setting_disabled(SETTING_CF_MEDIA)) {
         while ((call = call_group_get_next(info->group, call)) ) {
             streams = vector_iterator(call->streams);
+
             while ((stream = vector_iterator_next(&streams))) {
-                if (stream_get_count(stream)) {
+                if (stream->type == PACKET_RTP && stream_get_count(stream)) {
                     addr = stream->src;
                     addr.port = 0;
                     call_flow_column_add(ui, NULL, addr);
@@ -406,12 +403,10 @@ call_flow_draw_message(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     char delta[15] = { };
     int flowh, floww;
     char mediastr[40];
-    call_flow_column_t *column1, *column2;
     sip_msg_t *msg = arrow->item;
     vector_iter_t medias;
     int color = 0;
     int msglen;
-    int arrow_dir = CF_ARROW_RIGHT;
 
     // Get panel information
     info = call_flow_info(ui);
@@ -472,19 +467,20 @@ call_flow_draw_message(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     msglen = (strlen(method) > 24) ? 24 : strlen(method);
 
     // Get origin and destination column
-    column1 = call_flow_column_get(ui, callid, src);
-    column2 = call_flow_column_get(ui, callid, dst);
+    arrow->scolumn = call_flow_column_get(ui, callid, src);
+    arrow->dcolumn = call_flow_column_get(ui, callid, dst);
 
-    call_flow_column_t *tmp;
-    if (column1->colpos > column2->colpos) {
-        tmp = column1;
-        column1 = column2;
-        column2 = tmp;
+    // Determine start and end position of the arrow line
+    int arrow_dir, startpos, endpos;
+    if (arrow->scolumn->colpos < arrow->dcolumn->colpos) {
+        arrow_dir = CF_ARROW_RIGHT;
+        startpos = 20 + 30 * arrow->scolumn->colpos;
+        endpos = 20 + 30 * arrow->dcolumn->colpos;
+    } else {
         arrow_dir = CF_ARROW_LEFT;
+        startpos = 20 + 30 * arrow->dcolumn->colpos;
+        endpos = 20 + 30 * arrow->scolumn->colpos;
     }
-
-    int startpos = 20 + 30 * column1->colpos;
-    int endpos = 20 + 30 * column2->colpos;
     int distance = abs(endpos - startpos) - 3;
 
     // Highlight current message
@@ -545,13 +541,13 @@ call_flow_draw_message(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     // Write the arrow at the end of the message (two arros if this is a retrans)
     if (arrow_dir == CF_ARROW_RIGHT) {
         mvwaddch(flow_win, cline, endpos - 2, '>');
-        if (call_msg_is_retrans(msg)) {
+        if (msg->retrans) {
             mvwaddch(flow_win, cline, endpos - 3, '>');
             mvwaddch(flow_win, cline, endpos - 4, '>');
         }
     } else {
         mvwaddch(flow_win, cline, startpos + 2, '<');
-        if (call_msg_is_retrans(msg)) {
+        if (msg->retrans) {
             mvwaddch(flow_win, cline, startpos + 3, '<');
             mvwaddch(flow_win, cline, startpos + 4, '<');
         }
@@ -571,7 +567,14 @@ call_flow_draw_message(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     if (info->arrowtime) {
         if (arrow == call_flow_arrow_selected(ui))
             wattron(flow_win, COLOR_PAIR(CP_CYAN_ON_DEF));
-        mvwprintw(flow_win, cline, 2, "%s", msg_time);
+
+        if (arrow == vector_item(info->darrows, info->cur_arrow)) {
+            wattron(flow_win, A_BOLD);
+            mvwprintw(flow_win, cline, 2, "%s", msg_time);
+            wattroff(flow_win, A_BOLD);
+        } else {
+            mvwprintw(flow_win, cline, 2, "%s", msg_time);
+        }
 
         // Print delta from selected message
         if (!setting_has_value(SETTING_CF_SDP_INFO, "compressed")) {
@@ -606,10 +609,11 @@ call_flow_draw_rtp_stream(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     char text[50], time[20];
     int height, width;
     const char *callid;
-    address_t msg_src, msg_dst, stream_src, stream_dst;
-    call_flow_column_t *column1, *column2;
     rtp_stream_t *stream = arrow->item;
-    int arrow_dir = CF_ARROW_RIGHT;
+    sip_msg_t *msg;
+    sip_call_t *call;
+    call_flow_arrow_t *msgarrow;
+    address_t addr;
 
     // Get panel information
     info = call_flow_info(ui);
@@ -627,48 +631,85 @@ call_flow_draw_rtp_stream(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     if (cline > height + arrow->height)
         return 0;
 
-    // Get Message method (include extra info)
+    // Get arrow text
     sprintf(text, "RTP (%s) %d", stream_get_format(stream), stream_get_count(stream));
 
     // Get message data
-    callid = stream->media->msg->call->callid;
-    msg_src = stream->media->msg->packet->src;
-    msg_dst = stream->media->msg->packet->dst;
-    stream_src = stream->src;
-    stream_src.port = 0;
-    stream_dst = stream->dst;
-    stream_dst.port = 0;
+    call = stream->media->msg->call;
+    callid = call->callid;
 
-    // Get origin column for this stream.
-    // If we share the same Address from its setup SIP packet, use that column instead.
-    if (!strcmp(stream->src.ip, msg_src.ip)) {
-        column1 = call_flow_column_get(ui, callid, msg_src);
-    } else if (!strcmp(stream->src.ip, msg_dst.ip)) {
-        column1 = call_flow_column_get(ui, callid, msg_dst);
-    } else {
-        column1 = call_flow_column_get(ui, 0, stream_src);
+    /**
+     * This logic will try to use the same columns for the stream representation
+     * that are used in the SIP messages that configured the streams in their SDP
+     * if they share the same IP addresses.
+     */
+    // Message with Stream destination configured in SDP content
+    msg = stream->media->msg;
+
+    // If message and stream share the same IP address
+    if (address_equals(msg->packet->src, stream->dst)) {
+        // Reuse the msg arrow columns as destination column
+        if ((msgarrow = call_flow_arrow_find(ui, msg))) {
+            // Get origin and destination column
+            arrow->dcolumn = call_flow_column_get(ui, msg->call->callid, msg->packet->src);
+        }
     }
 
-    // Get destination column for this stream.
-    // If we share the same Address from its setup SIP packet, use that column instead.
-    if (!strcmp(stream->dst.ip, msg_dst.ip)) {
-        column2 = call_flow_column_get(ui, callid, msg_dst);
-    } else if (!strcmp(stream->dst.ip, msg_src.ip)) {
-        column2 = call_flow_column_get(ui, callid, msg_src);
-    } else {
-        column2 = call_flow_column_get(ui, 0, stream_dst);
+    // fallback: Just use any column that have the destination IP printed
+    if (!arrow->dcolumn) {
+        // FIXME figure a better way to find ignoring port :(
+        addr = stream->dst; addr.port = 0;
+        arrow->dcolumn = call_flow_column_get(ui, 0, addr);
     }
 
-    call_flow_column_t *tmp;
-    if (column1->colpos > column2->colpos) {
-        tmp = column1;
-        column1 = column2;
-        column2 = tmp;
+    /**
+     * For source address of the stream, first try to find a message that have
+     * the stream source configured in their SDP as destination and then apply
+     * the same previous logic address: if IP address matches, reuse message
+     * column, othwerise any column with the source IP will be used.
+     */
+    // Message with Stream source configured in SDP content
+    msg = call_msg_with_media(call, stream->src);
+
+    // Try to find a message with configured SDP matching the source of this stream
+    if (msg && address_equals(msg->packet->src, stream->src)) {
+        // Reuse the msg arrow columns as destination column
+        if ((msgarrow = call_flow_arrow_find(ui, msg))) {
+            // Get origin and destination column
+            arrow->scolumn = call_flow_column_get(ui, msg->call->callid, msg->packet->src);
+        }
+    }
+
+    // Prefer message that configured this stream rather than any column
+    if (!arrow->scolumn) {
+        msg = stream->media->msg;
+        // If message and stream share the same IP address
+        if (address_equals(msg->packet->dst, stream->src)) {
+            // Reuse the msg arrow columns as destination column
+            if ((msgarrow = call_flow_arrow_find(ui, msg))) {
+                arrow->scolumn = call_flow_column_get(ui, msg->call->callid, msg->packet->dst);
+            }
+        }
+    }
+
+    // fallback: Just use any column that have the soruce IP printed
+    if (!arrow->scolumn) {
+        // FIXME figure a better way to find ignoring port :(
+        addr = stream->src; addr.port = 0;
+        arrow->scolumn = call_flow_column_get(ui, 0, addr);
+    }
+
+    // Determine start and end position of the arrow line
+    int arrow_dir, startpos, endpos;
+    if (arrow->scolumn->colpos < arrow->dcolumn->colpos) {
+        arrow_dir = CF_ARROW_RIGHT;
+        startpos = 20 + 30 * arrow->scolumn->colpos;
+        endpos = 20 + 30 * arrow->dcolumn->colpos;
+    } else {
         arrow_dir = CF_ARROW_LEFT;
+        startpos = 20 + 30 * arrow->dcolumn->colpos;
+        endpos = 20 + 30 * arrow->scolumn->colpos;
     }
-
-    int startpos = 20 + 30 * column1->colpos;
-    int endpos = 20 + 30 * column2->colpos;
     int distance = 0;
 
     if (startpos != endpos) {
@@ -707,16 +748,23 @@ call_flow_draw_rtp_stream(ui_t *ui, call_flow_arrow_t *arrow, int cline)
         }
     }
 
+    // Check if displayed stream is active
+    int active = stream_is_active(stream);
+
     // Clear the line
     mvwprintw(win, cline, startpos + 2, "%*s", distance, "");
-    // Draw method
+    // Draw RTP arrow text
     mvwprintw(win, cline, startpos + (distance) / 2 - strlen(text) / 2 + 2, "%s", text);
 
     if (!setting_has_value(SETTING_CF_SDP_INFO, "compressed"))
         cline++;
 
     // Draw line between columns
-    mvwhline(win, cline, startpos + 2, ACS_HLINE, distance);
+    if (active)
+        mvwhline(win, cline, startpos + 2, '-', distance);
+    else
+        mvwhline(win, cline, startpos + 2, ACS_HLINE, distance);
+
     // Write the arrow at the end of the message (two arrows if this is a retrans)
     if (arrow_dir == CF_ARROW_RIGHT) {
         if (!setting_has_value(SETTING_CF_SDP_INFO, "compressed")) {
@@ -724,7 +772,7 @@ call_flow_draw_rtp_stream(ui_t *ui, call_flow_arrow_t *arrow, int cline)
             mvwprintw(win, cline, endpos, "%d", stream->dst.port);
         }
         mvwaddch(win, cline, endpos - 2, '>');
-        if (arrow->rtp_count != stream_get_count(stream)) {
+        if (active) {
             arrow->rtp_count = stream_get_count(stream);
             arrow->rtp_ind_pos = (arrow->rtp_ind_pos + 1) % distance;
             mvwaddch(win, cline, startpos + arrow->rtp_ind_pos + 2, '>');
@@ -735,7 +783,7 @@ call_flow_draw_rtp_stream(ui_t *ui, call_flow_arrow_t *arrow, int cline)
             mvwprintw(win, cline, startpos - 4, "%d", stream->dst.port);
         }
         mvwaddch(win, cline, startpos + 2, '<');
-        if (arrow->rtp_count != stream_get_count(stream)) {
+        if (active) {
             arrow->rtp_count = stream_get_count(stream);
             arrow->rtp_ind_pos = (arrow->rtp_ind_pos + 1) % distance;
             mvwaddch(win, cline, endpos - arrow->rtp_ind_pos - 2, '<');
@@ -750,7 +798,14 @@ call_flow_draw_rtp_stream(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     // Print timestamp
     if (info->arrowtime) {
         timeval_to_time(stream->time, time);
-        mvwprintw(win, cline, 2, "%s", time);
+        if (arrow == vector_item(info->darrows, info->cur_arrow)) {
+            wattron(win, A_BOLD);
+            mvwprintw(win, cline, 2, "%s", time);
+            wattroff(win, A_BOLD);
+        } else {
+            mvwprintw(win, cline, 2, "%s", time);
+        }
+
     }
 
     return arrow->height;
@@ -780,6 +835,8 @@ int
 call_flow_arrow_height(ui_t *ui, const call_flow_arrow_t *arrow)
 {
     if (arrow->type == CF_ARROW_SIP) {
+        if (setting_enabled(SETTING_CF_ONLYMEDIA))
+            return 0;
         if (setting_has_value(SETTING_CF_SDP_INFO, "compressed"))
             return 1;
         if (!msg_has_sdp(arrow->item))
@@ -793,6 +850,8 @@ call_flow_arrow_height(ui_t *ui, const call_flow_arrow_t *arrow)
     } else if (arrow->type == CF_ARROW_RTP || arrow->type == CF_ARROW_RTCP) {
         if (setting_has_value(SETTING_CF_SDP_INFO, "compressed"))
             return 1;
+        if (setting_disabled(SETTING_CF_MEDIA))
+            return 0;
         return 2;
     }
 
@@ -885,7 +944,7 @@ call_flow_draw_raw(ui_t *ui, sip_msg_t *msg)
         info->raw_win = raw_win = newwin(raw_height, raw_width, 0, 0);
     }
 
-    // Draw raw box lines
+    // Draw raw box li
     wattron(ui->win, COLOR_PAIR(CP_BLUE_ON_DEF));
     mvwvline(ui->win, 1, ui->width - raw_width - 2, ACS_VLINE, ui->height - 2);
     wattroff(ui->win, COLOR_PAIR(CP_BLUE_ON_DEF));
@@ -903,6 +962,11 @@ call_flow_draw_raw(ui_t *ui, sip_msg_t *msg)
 int
 call_flow_draw_raw_rtcp(ui_t *ui, rtp_stream_t *stream)
 {
+    /**
+     * TODO This is too experimental to even display it
+     */
+    return 0;
+
     call_flow_info_t *info;
     WINDOW *raw_win;
     int raw_width, raw_height;
@@ -1058,6 +1122,10 @@ call_flow_handle_key(ui_t *ui, int key)
             case ACTION_SDP_INFO:
                 setting_toggle(SETTING_CF_SDP_INFO);
                 break;
+            case ACTION_ONLY_MEDIA:
+                setting_toggle(SETTING_CF_ONLYMEDIA);
+                call_flow_set_group(info->group);
+                break;
             case ACTION_TOGGLE_MEDIA:
                 setting_toggle(SETTING_CF_MEDIA);
                 // Force reload arrows
@@ -1287,6 +1355,7 @@ call_flow_move(ui_t *ui, int arrowindex)
     call_flow_info_t *info;
     call_flow_arrow_t *arrow;
     int flowh;
+    int curh = 0;
 
     // Get panel info
     if (!(info = call_flow_info(ui)))
@@ -1329,15 +1398,28 @@ call_flow_move(ui_t *ui, int arrowindex)
     }
 
     // Update the first displayed arrow
-    if (info->cur_arrow < info->first_arrow) {
+    if (info->cur_arrow <= info->first_arrow) {
         info->first_arrow = info->cur_arrow;
-    } else if (info->cur_arrow - info->first_arrow >= flowh/2) {
-        // If we are out of the bottom of the displayed list
-        // refresh it starting in the next call
-        info->first_arrow = info->cur_arrow - flowh/2 + 1;
+    } else {
+        // Draw the scrollbar
+        vector_iterator_set_current(&it, info->first_arrow - 1);
+        while ((arrow = vector_iterator_next(&it))) {
+            // Increase current arrow height position
+            curh += call_flow_arrow_height(ui, arrow);
+            // If we have reached current arrow
+            if (vector_iterator_current(&it) == info->cur_arrow) {
+                if (curh > flowh) {
+                    // Go to the next first arrow and check if current arrow
+                    // is still out of bottom bounds
+                    info->first_arrow++;
+                    vector_iterator_set_current(&it, info->first_arrow - 1);
+                    curh = 0;
+                } else {
+                    break;
+                }
+            }
+        }
     }
-
-
 }
 
 call_flow_arrow_t *
@@ -1413,12 +1495,18 @@ call_flow_arrow_filter(void *item)
     call_flow_arrow_t *arrow = (call_flow_arrow_t *) item;
 
     // SIP arrows are never filtered
-    if (arrow->type == CF_ARROW_SIP)
+    if (arrow->type == CF_ARROW_SIP && setting_disabled(SETTING_CF_ONLYMEDIA))
         return 1;
 
     // RTP arrows are only displayed when requested
-    if (arrow->type == CF_ARROW_RTP && setting_enabled(SETTING_CF_MEDIA))
-        return 1;
+    if (arrow->type == CF_ARROW_RTP) {
+        // Display all streams
+        if (setting_enabled(SETTING_CF_MEDIA))
+            return 1;
+        // Otherwise only show active streams
+        if (setting_has_value(SETTING_CF_MEDIA, SETTING_ACTIVE))
+            return stream_is_active(arrow->item);
+    }
 
     // Rest of the arrows are never displayed
     return 0;
