@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2013-2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2013-2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -59,12 +59,6 @@ media_destroyer(void *item)
     sng_free(media);
 }
 
-void
-media_set_port(sdp_media_t *media, u_short port)
-{
-    media->port = port;
-}
-
 void
 media_set_type(sdp_media_t *media, const char *type)
 {
@@ -72,19 +66,19 @@ media_set_type(sdp_media_t *media, const char *type)
 }
 
 void
-media_set_address(sdp_media_t *media, const char *address)
+media_set_address(sdp_media_t *media, address_t addr)
 {
-    strcpy(media->address, address);
+    media->address = addr;
 }
 
 void
-media_set_prefered_format(sdp_media_t *media, u_int code)
+media_set_prefered_format(sdp_media_t *media, uint32_t code)
 {
     media->fmtcode = code;
 }
 
 void
-media_add_format(sdp_media_t *media, u_int code, const char *format)
+media_add_format(sdp_media_t *media, uint32_t code, const char *format)
 {
     sdp_media_fmt_t *fmt;
 
@@ -97,25 +91,7 @@ media_add_format(sdp_media_t *media, u_int code, const char *format)
 }
 
 const char *
-media_get_address(sdp_media_t *media)
-{
-    return media->address;
-}
-
-u_short
-media_get_port(sdp_media_t *media)
-{
-    return media->port;
-}
-
-const char *
-media_get_type(sdp_media_t *media)
-{
-    return media->type;
-}
-
-const char *
-media_get_format(sdp_media_t *media, u_int code)
+media_get_format(sdp_media_t *media, uint32_t code)
 {
     sdp_media_fmt_t *format;
     vector_iter_t iter;
