@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2013-2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2013-2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 #include <sys/types.h>
 #include "capture.h"
 
+#define MEDIATYPELEN 15
+
 //! Shorter declaration of sdp_media structure
 typedef struct sdp_media sdp_media_t;
 //! Shorter declaration of sdp_media_fmt structure
@@ -43,17 +45,16 @@ struct sip_msg;
 struct sip_call;
 
 struct sdp_media_fmt {
-    u_int id;
+    uint32_t id;
     char format[50];
 };
 
 
 struct sdp_media {
     //! SDP Addresses information
-    char address[ADDRESSLEN];
-    u_short port;
-    char type[10];
-    u_int fmtcode;
+    address_t address;
+    char type[MEDIATYPELEN];
+    uint32_t fmtcode;
     //! List of described formats in this media
     vector_t *formats;
     //! Message with this SDP content
@@ -77,32 +78,23 @@ media_create(struct sip_msg *msg);
 void
 media_destroyer(void *item);
 
-void
-media_set_port(sdp_media_t *media, u_short port);
-
 void
 media_set_type(sdp_media_t *media, const char *type);
 
 void
-media_set_address(sdp_media_t *media, const char *address);
+media_set_address(sdp_media_t *media, address_t addr);
 
 void
-media_set_prefered_format(sdp_media_t *media, u_int code);
+media_set_prefered_format(sdp_media_t *media, uint32_t code);
 
 void
-media_add_format(sdp_media_t *media, u_int code, const char *format);
-
-const char *
-media_get_address(sdp_media_t *media);
-
-u_short
-media_get_port(sdp_media_t *media);
+media_add_format(sdp_media_t *media, uint32_t code, const char *format);
 
 const char *
 media_get_type(sdp_media_t *media);
 
 const char *
-media_get_format(sdp_media_t *media, u_int code);
+media_get_format(sdp_media_t *media, uint32_t code);
 
 const char *
 media_get_prefered_format(sdp_media_t *media);
