@@ -26,9 +26,11 @@
  * @brief Source code of functions defined in option.h
  *
  */
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <time.h>
 #include "keybinding.h"
 #include "option.h"
@@ -50,9 +52,14 @@ init_options()
     // Custom user conf file
     char userconf[128];
     char *home = getenv("HOME");
+    char pwd[MAX_SETTING_LEN];
+
+    // Defualt savepath is current directory
+    if (getcwd(pwd, MAX_SETTING_LEN)) {
+        setting_set_value(SETTING_SAVEPATH, pwd);
+    }
 
     // Initialize settings
-    setting_set_value(SETTING_SAVEPATH, home);
     setting_set_value(SETTING_FILTER_METHODS, "REGISTER,INVITE,SUBSCRIBE,NOTIFY,OPTIONS,PUBLISH,MESSAGE");
 
     // Add Call list column options
@@ -104,7 +111,7 @@ read_options(const char *fname)
             continue;
 
         // Get configuration option from setting line
-        if (sscanf(line, "%s %s %s", type, option, value) == 3) {
+        if (sscanf(line, "%s %s %[^\t\n]", type, option, value) == 3) {
             if (!strcasecmp(type, "set")) {
                 if ((id = setting_id(option)) >= 0) {
                     setting_set_value(id, value);
