@@ -29,6 +29,9 @@
  *
  */
 
+#include "config.h"
+#include <stddef.h>
+#include <time.h>
 #include "rtp.h"
 #include "sip.h"
 #include "vector.h"
@@ -100,6 +103,7 @@ stream_add_packet(rtp_stream_t *stream, packet_t *packet)
     if (stream->pktcnt == 0)
         stream->time = packet_time(packet);
 
+    stream->lasttm = (int) time(NULL);
     stream->pktcnt++;
 }
 
@@ -176,16 +180,7 @@ rtp_check_packet(packet_t *packet)
     src = packet->src;
     dst = packet->dst;
 
-    // Check if we have at least RTP type
-    if ((int32_t) size < 2)
-        return NULL;
-
-    // Check RTP version
-    if (RTP_VERSION(*payload) != RTP_VERSION_RFC1889)
-        return NULL;
-
-    // RTP: even, RTCP: odd
-    if (((packet->dst.port % 2) == 0)) {
+    if (data_is_rtp(payload, size) == 0) {
 
         // Get RTP payload type
         format = RTP_PAYLOAD_TYPE(*(payload + 1));
@@ -210,18 +205,49 @@ rtp_check_packet(packet_t *packet)
         if (!(stream_is_complete(stream))) {
             stream_complete(stream, src);
             stream_set_format(stream, format);
+
+            /**
+             * TODO This is a mess. Rework required
+             *
+             * This logic tries to handle a common problem when SDP address and RTP address
+             * doesn't match. In some cases one endpoint waits until RTP data is sent to its
+             * configured port in SDP and replies its RTP to the source ignoring what the other
+             * endpoint has configured in its SDP.
+             *
+             * For such cases, we create streams 'on the fly', when a stream is completed (the
+             * first time its source address is filled), a new stream is created with the
+             * opposite src and dst.
+             *
+             * BUT, there are some cases when this 'reverse' stream should not be created:
+             *  - When there already exists a stream with that setup
+             *  - When there exists an incomplete stream with that destination (and still no source)
+             *  - ...
+             *
+             */
+
             // Check if an stream in the opposite direction exists
             if (!(reverse = rtp_find_call_stream(stream->media->msg->call, stream->dst, stream->src))) {
                 reverse = stream_create(stream->media, stream->src, PACKET_RTP);
                 stream_complete(reverse, stream->dst);
                 stream_set_format(reverse, format);
                 call_add_stream(msg_get_call(stream->media->msg), reverse);
+            } else {
+                // If the reverse stream has other source configured
+                if (reverse->src.port && !addressport_equals(stream->src, reverse->src)) {
+                    if (!(reverse = rtp_find_call_exact_stream(stream->media->msg->call, stream->dst, stream->src))) {
+                        // Create a new reverse stream
+                        reverse = stream_create(stream->media, stream->src, PACKET_RTP);
+                        stream_complete(reverse, stream->dst);
+                        stream_set_format(reverse, format);
+                        call_add_stream(msg_get_call(stream->media->msg), reverse);
+                    }
+                }
             }
         }
 
         // Add packet to stream
         stream_add_packet(stream, packet);
-    } else {
+    } else if (data_is_rtcp(payload, size) == 0) {
         // Find the matching stream
         if ((stream = rtp_find_stream(src, dst))) {
 
@@ -294,6 +320,8 @@ rtp_check_packet(packet_t *packet)
             stream_complete(stream, src);
             stream_add_packet(stream, packet);
         }
+    } else {
+      return NULL;
     }
 
     return stream;
@@ -397,14 +425,29 @@ rtp_find_call_stream(struct sip_call *call, address_t src, address_t dst)
         }
     }
 
-    // Try to look for an incomplete stream with this destination
+    // Try to look for a complete stream with this destination
     if (src.port) {
-        vector_iterator_set_last(&it);
-        while ((stream = vector_iterator_prev(&it))) {
-            if (addressport_equals(src, stream->src) &&
-                addressport_equals(dst, stream->dst)) {
-                return stream;
-            }
+        return rtp_find_call_exact_stream(call, src, dst);
+    }
+
+    // Nothing found
+    return NULL;
+}
+
+rtp_stream_t *
+rtp_find_call_exact_stream(struct sip_call *call, address_t src, address_t dst)
+{
+    rtp_stream_t *stream;
+    vector_iter_t it;
+
+    // Create an iterator for call streams
+    it = vector_iterator(call->streams);
+
+    vector_iterator_set_last(&it);
+    while ((stream = vector_iterator_prev(&it))) {
+        if (addressport_equals(src, stream->src) &&
+            addressport_equals(dst, stream->dst)) {
+            return stream;
         }
     }
 
@@ -419,6 +462,10 @@ stream_is_older(rtp_stream_t *one, rtp_stream_t *two)
     if (!two)
         return 1;
 
+    // No, you are not older than yourself
+    if (one == two)
+        return 0;
+
     // Otherwise
     return timeval_is_older(one->time, two->time);
 }
@@ -428,3 +475,41 @@ stream_is_complete(rtp_stream_t *stream)
 {
     return (stream->pktcnt != 0);
 }
+
+int
+stream_is_active(rtp_stream_t *stream)
+{
+    return ((int) time(NULL) - stream->lasttm <= STREAM_INACTIVE_SECS);
+}
+
+int
+data_is_rtp(u_char *data, uint32_t len)
+{
+    u_char pt = RTP_PAYLOAD_TYPE(*(data + 1));
+
+    if ((len >= RTP_HDR_LENGTH) &&
+        (RTP_VERSION(*data) == RTP_VERSION_RFC1889) &&
+        (data[0] > 127 && data[0] < 192) &&
+        (pt <= 64 || pt >= 96)) {
+        return 0;
+    }
+
+    // Not a RTP packet
+    return 1;
+}
+
+int
+data_is_rtcp(u_char *data, uint32_t len)
+{
+    struct rtcp_hdr_generic *hdr = (struct rtcp_hdr_generic*) data;
+
+    if ((len >= RTCP_HDR_LENGTH) &&
+        (RTP_VERSION(*data) == RTP_VERSION_RFC1889) &&
+        (data[0] > 127 && data[0] < 192) &&
+        (hdr->type >= 192 && hdr->type <= 223)) {
+        return 0;
+    }
+
+    // Not a RTCP packet
+    return 1;
+}
