@@ -56,12 +56,14 @@ typedef struct setting_option setting_t;
 #define SETTING_ENUM_SDP_INFO    (const char *[]){ "off", "first", "full", "compressed", NULL}
 #define SETTING_ENUM_STORAGE     (const char *[]){ "none", "memory", NULL }
 #define SETTING_ENUM_HEPVERSION  (const char *[]){ "2", "3", NULL }
+#define SETTING_ENUM_MEDIA       (const char *[]){ "off", "on", "active", NULL }
 
 //! Other useful defines
 #define SETTING_ON  "on"
 #define SETTING_OFF "off"
 #define SETTING_YES "yes"
 #define SETTING_NO  "no"
+#define SETTING_ACTIVE "active"
 
 
 //! Available setting Options
@@ -87,6 +89,8 @@ enum setting_id {
     SETTING_CL_SCROLLSTEP,
     SETTING_CL_COLORATTR,
     SETTING_CL_AUTOSCROLL,
+    SETTING_CL_SORTFIELD,
+    SETTING_CL_SORTORDER,
     SETTING_CF_FORCERAW,
     SETTING_CF_RAWMINWIDTH,
     SETTING_CF_RAWFIXEDWIDTH,
@@ -96,6 +100,7 @@ enum setting_id {
     SETTING_CF_LOCALHIGHLIGHT,
     SETTING_CF_SDP_INFO,
     SETTING_CF_MEDIA,
+    SETTING_CF_ONLYMEDIA,
     SETTING_CF_DELTA,
     SETTING_CR_SCROLLSTEP,
     SETTING_FILTER_PAYLOAD,
