@@ -40,6 +40,7 @@
 #endif
 #include "sip_call.h"
 #include "vector.h"
+#include "hash.h"
 
 #define MAX_SIP_PAYLOAD 10240
 
@@ -122,6 +123,10 @@ struct sip_call_list {
     bool changed;
     //! Sort call list following this options
     sip_sort_t sort;
+    //! Last created id
+    int last_index;
+    //! Call-Ids hash table
+    htable_t *callids;
 
     // Max call limit
     int limit;
@@ -152,6 +157,8 @@ struct sip_call_list {
     regex_t reg_valid;
     regex_t reg_cl;
     regex_t reg_body;
+    regex_t reg_reason;
+    regex_t reg_warning;
 };
 
 /**
@@ -261,6 +268,15 @@ sip_calls_iterator();
 vector_iter_t
 sip_active_calls_iterator();
 
+/**
+ * @brief Return if a call is in active's call vector
+ *
+ * @param call Call to be searched
+ * @return TRUE if call is active, FALSE otherwise
+ */
+bool
+sip_call_is_active(sip_call_t *call);
+
 /**
  * @brief Return the call list
  */
@@ -295,6 +311,18 @@ sip_find_by_index(int index);
 sip_call_t *
 sip_find_by_callid(const char *callid);
 
+
+/**
+ * @brief Parse extra fields only for dialogs strarting with invite
+ *
+ * @note This function assumes the msg is already part of a call
+ *
+ * @param msg SIP message structure
+ * @param payload SIP message payload
+ */
+void
+sip_parse_extra_headers(sip_msg_t *msg, const u_char *payload);
+
 /**
  * @brief Remove al calls
  *
@@ -304,6 +332,15 @@ sip_find_by_callid(const char *callid);
 void
 sip_calls_clear();
 
+/**
+ * @brief Remove first call in the call list
+ *
+ * This function removes the first call in the calls vector avoiding
+ * reaching the capture limit.
+ */
+void
+sip_calls_rotate();
+
 /**
  * @brief Get message Request/Response code
  *
@@ -362,7 +399,7 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload);
 /**
  * @brief Set Capture Matching expression
  *
- * @param expr String containing matching expreson
+ * @param expr String containing matching expression
  * @param insensitive 1 for case insensitive matching
  * @param invert 1 for reverse matching
  * @return 0 if expresion is valid, 1 otherwise
@@ -370,6 +407,14 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload);
 int
 sip_set_match_expression(const char *expr, int insensitive, int invert);
 
+/**
+ * @brief Get Capture Matching expression
+ *
+ * @return String containing matching expression
+ */
+const char *
+sip_get_match_expression();
+
 /**
  * @brief Checks if a given payload matches expression
  *
