@@ -81,6 +81,10 @@ enum sip_attr_id {
     SIP_ATTR_CONVDUR,
     //! Total call duration
     SIP_ATTR_TOTALDUR,
+    //! Text from SIP Reason header
+    SIP_ATTR_REASON_TXT,
+    //! Warning Header
+    SIP_ATTR_WARNING,
     //! SIP Attribute count
     SIP_ATTR_COUNT
 };
@@ -183,7 +187,7 @@ sip_attr_get_width(enum sip_attr_id id);
  * @param name Attribut name
  * @return Attribute id or -1 if not found
  */
-enum sip_attr_id
+int
 sip_attr_from_name(const char *name);
 
 /**
