@@ -83,6 +83,7 @@ call_destroy(sip_call_t *call)
     // Deallocate call memory
     sng_free(call->callid);
     sng_free(call->xcallid);
+    sng_free(call->reasontxt);
     sng_free(call);
 }
 
@@ -149,8 +150,8 @@ call_is_invite(sip_call_t *call)
     return 0;
 }
 
-int
-call_msg_is_retrans(sip_msg_t *msg)
+void
+call_msg_retrans_check(sip_msg_t *msg)
 {
     sip_msg_t *prev = NULL;
     vector_iter_t it;
@@ -164,9 +165,33 @@ call_msg_is_retrans(sip_msg_t *msg)
             break;
     }
 
-    return (prev && !strcasecmp(msg_get_payload(msg), msg_get_payload(prev)));
+    // Store the flag that determines if message is retrans
+    if (prev && !strcasecmp(msg_get_payload(msg), msg_get_payload(prev))) {
+        msg->retrans = prev;
+    }
 }
 
+sip_msg_t *
+call_msg_with_media(sip_call_t *call, address_t dst)
+{
+    sip_msg_t *msg;
+    sdp_media_t *media;
+    vector_iter_t itmsg;
+    vector_iter_t itmedia;
+
+    // Get message with media address configured in given dst
+    itmsg = vector_iterator(call->msgs);
+    while ((msg = vector_iterator_next(&itmsg))) {
+        itmedia = vector_iterator(msg->medias);
+        while ((media = vector_iterator_next(&itmedia))) {
+            if (addressport_equals(dst, media->address)) {
+                return msg;
+            }
+        }
+    }
+
+    return NULL;
+}
 
 void
 call_update_state(sip_call_t *call, sip_msg_t *msg)
@@ -193,9 +218,15 @@ call_update_state(sip_call_t *call, sip_msg_t *msg)
             } else if (reqresp == SIP_METHOD_CANCEL) {
                 // Alice is not in the mood
                 call->state = SIP_CALLSTATE_CANCELLED;
-            } else if (reqresp > 400) {
+            } else if ((reqresp == 480) || (reqresp == 486)) {
+                // Bob is busy
+                call->state = SIP_CALLSTATE_BUSY;
+            } else if (reqresp > 400 && call->invitecseq == msg->cseq) {
                 // Bob is not in the mood
                 call->state = SIP_CALLSTATE_REJECTED;
+            } else if (reqresp > 300) {
+                // Bob has diversion
+                call->state = SIP_CALLSTATE_DIVERTED;
             }
         } else if (call->state == SIP_CALLSTATE_INCALL) {
             if (reqresp == SIP_METHOD_BYE) {
@@ -253,6 +284,14 @@ call_get_attribute(sip_call_t *call, enum sip_attr_id id, char *value)
             last = vector_last(call->msgs);
             timeval_to_duration(msg_get_time(first), msg_get_time(last), value);
             break;
+        case SIP_ATTR_REASON_TXT:
+            if (call->reasontxt)
+                sprintf(value, "%s", call->reasontxt);
+            break;
+        case SIP_ATTR_WARNING:
+            if (call->warning)
+                sprintf(value, "%d", call->warning);
+            break;
         default:
             return msg_get_attribute(vector_first(call->msgs), id, value);
             break;
@@ -273,6 +312,10 @@ call_state_to_str(int state)
             return "CANCELLED";
         case SIP_CALLSTATE_REJECTED:
             return "REJECTED";
+        case SIP_CALLSTATE_BUSY:
+            return "BUSY";
+        case SIP_CALLSTATE_DIVERTED:
+            return "DIVERTED";
         case SIP_CALLSTATE_COMPLETED:
             return "COMPLETED";
     }
