@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -48,6 +48,12 @@ sng_malloc(size_t size);
 void
 sng_free(void *ptr);
 
+/*
+ * @brief Generic implementation of basename
+ */
+char *
+sng_basename(const char *name);
+
 /**
  * @brief Compare two timeval structures
  *
