@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #define __SNGREP_VECTOR_H_
 
 #include "config.h"
-#include <sys/types.h>
+#include <stdint.h>
 
 //! Shorter declaration of vector structure
 typedef struct vector vector_t;
@@ -44,11 +44,11 @@ typedef struct vector_iter vector_iter_t;
  */
 struct vector {
     //! Number of elements in list
-    u_int count;
+    uint32_t count;
     //! Total space in list (available + elements)
-    u_int limit;
+    uint32_t limit;
     //! Number of new spaces to be reallocated
-    u_short step;
+    uint8_t step;
     //! Elements of the vector
     void **list;
     //! Function to destroy one item
@@ -60,6 +60,8 @@ struct vector {
 struct vector_iter {
     //! Last requested position
     int current;
+    //! Last vector position
+    int current_vector;
     //! Vector that's being iterated
     vector_t *vector;
     //! Filter iterator results using this func
@@ -81,6 +83,12 @@ vector_create(int limit, int step);
 void
 vector_destroy(vector_t *vector);
 
+/**
+ * @brief Free all vector items and memory
+ */
+void
+vector_destroy_items(vector_t *vector);
+
 /**
  * @brief Clone a vector container
  *
@@ -94,6 +102,13 @@ vector_destroy(vector_t *vector);
 vector_t *
 vector_clone(vector_t *original);
 
+/**
+ * @brief Copy filtered elements to a new vector
+ *
+ */
+vector_t *
+vector_copy_if(vector_t *original, int (*filter)(void *item));
+
 /**
  * @brief Remove all items of vector
  *
