@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  *
  * Basic Message diff testing
  */
-const int keys[] =
+const char keys[] =
         {
          /* Select some dialog */
          107, 107, 107, 32, 10,
